{(C) 2000 Elk-Software                              }
{Datum    : 30.01.2000                              }
{Autor    : Markus Humm                             }
{Zweck    : das API zur Verwendung von TTFs         }
{Plattform: PROTECTED MODE                          }
{nderung : 12.02.2000                              }
UNIT TTFAPI;

INTERFACE

uses crt;

  FUNCTION  ASCII2UNICODE(ascii:BYTE):WORD;
  FUNCTION  INIT_TTFAPI:WORD;
  PROCEDURE SET_TTFPARAMS(filename:STRING; pt_size:WORD; antialias:BOOLEAN);
  FUNCTION  OPEN_FONT(filename:STRING):WORD;
  PROCEDURE CLOSE_FONT;
  PROCEDURE DEINIT_TTFAPI;
  PROCEDURE OUTTEXTXY(x,y:INTEGER; s:STRING);

VAR dateiname:STRING;  {Name der Schriftdatei}
    loaded   :BOOLEAN; {wurde eine Schrift korrekt geladen?}

IMPLEMENTATION

USES COMMON,GMain,GEvents,GDriver,FreeType;

CONST Precis            = 64;
      Precis2           = Precis div 2;
      PrecisAux         = 1024;
      Profile_Buff_Size = 32000;
      Max_Files         = 1024;

VAR   face     : TT_Face;
      instance : TT_Instance;
      glyph    : TT_Glyph;

      metrics  : TT_Glyph_Metrics;
      imetrics : TT_Instance_Metrics;

      props    : TT_Face_Properties;

      ymin, ymax, xmax, xmin, xsize : LONGINT;
      res, old_res                  : INT;

      numPoints, numContours : INT;

      Bit : TT_Raster_Map;

      Rotation : INT;  (* Angle modulo 1024 *)

      num_glyphs : INT;

      error      : TT_Error;
      gray_level : BOOLEAN;

      display_outline : BOOLEAN;
      hint_glyph      : BOOLEAN;
      scan_type       : BYTE;

      old_glyph : INT;
      cur_glyph : INT;

      scale_shift : INT;

      grayLines : array[0..2048] of BYTE;

      charmap  :TT_CHARMAP;


{Autor: Markus Humm                                       }
{Datum: 30.1.2000                                         }
{Zweck: wandelt ein gegebenes Asciizeichen in die ent-    }
{       sprechende Unicode Nr. um                         }
{IN   : das ASCII-Zeichen (Codepage 437)                  }
{OUT  : die zugehrige Unicode Nr., oder 0 fr nicht      }
{       implementierte Zeichen (zumeist Zeichen die nicht }
{       ohne weiteres mit der Tastatur eingegeben werden  }
{       knnen)                                           }
FUNCTION ASCII2UNICODE(ascii:BYTE):WORD;
BEGIN
  IF ascii in [32..126] THEN
  BEGIN
    ASCII2UNICODE:=ascii-12;
  END
  ELSE
  BEGIN
    CASE ascii OF
       20: ASCII2UNICODE:=$B6;
       21: ASCII2UNICODE:=$A7;
      128: ASCII2UNICODE:=$C7;
      129: ASCII2UNICODE:=$FC;
      130: ASCII2UNICODE:=$E9;
      131: ASCII2UNICODE:=$E2;
      132: ASCII2UNICODE:=$E4;
      133: ASCII2UNICODE:=$E0;
      134: ASCII2UNICODE:=$E5;
      135: ASCII2UNICODE:=$E7;
      136: ASCII2UNICODE:=$EA;
      138: ASCII2UNICODE:=$E8;
      139: ASCII2UNICODE:=$EF;
      140: ASCII2UNICODE:=$EE;
      141: ASCII2UNICODE:=$EC;
      142: ASCII2UNICODE:=$C4;
      143: ASCII2UNICODE:=$C5;
      144: ASCII2UNICODE:=$C9;
      145: ASCII2UNICODE:=$E6;
      146: ASCII2UNICODE:=$C6;
      147: ASCII2UNICODE:=$F4;
      148: ASCII2UNICODE:=$F6;
      149: ASCII2UNICODE:=$F2;
      150: ASCII2UNICODE:=$FB;
      151: ASCII2UNICODE:=$F9;
      152: ASCII2UNICODE:=$FF;
      153: ASCII2UNICODE:=$D6;
      154: ASCII2UNICODE:=$DC;
      156: ASCII2UNICODE:=$A3;
      157: ASCII2UNICODE:=$A5;
      161: ASCII2UNICODE:=$ED;
      162: ASCII2UNICODE:=$F3;
      163: ASCII2UNICODE:=$FA;
      164: ASCII2UNICODE:=$F1;
      165: ASCII2UNICODE:=$D1;
      166: ASCII2UNICODE:=$AA;
      167: ASCII2UNICODE:=$BA;
      168: ASCII2UNICODE:=$BF;
      170: ASCII2UNICODE:=$AC;
      171: ASCII2UNICODE:=$BD;
      172: ASCII2UNICODE:=$BC;
      173: ASCII2UNICODE:=$A1;
      174: ASCII2UNICODE:=$AB;
      175: ASCII2UNICODE:=$BB;
      225: ASCII2UNICODE:=$DF;
      230: ASCII2UNICODE:=$B5;
      235: ASCII2UNICODE:=$F0;
      241: ASCII2UNICODE:=$B1;
      246: ASCII2UNICODE:=$F7;
      248: ASCII2UNICODE:=$B0;
      250: ASCII2UNICODE:=$B7;
      252: ASCII2UNICODE:=$207F;
      253: ASCII2UNICODE:=$B2;
    END;
  END;
END;

{Autor: Markus Humm                                       }
{Datum: 12.02.2000                                        }
{Zweck: initialisiert das True Type API                   }
{IN   : nichts                                            }
{OUT  : Fehlercode                                        }
FUNCTION INIT_TTFAPI:WORD;

  {Autor: FREETYPE                                          }
  {Datum: 12.02.2000                                        }
  {Zweck: initialisiert ein paar Variablen                  }
  {IN   : nichts                                            }
  {OUT  : nichts                                            }
  {=> mu noch gendert werden auf unser GFX-API !!!!!!!!!!!}
  PROCEDURE Set_Raster_Area;
  BEGIN
    Bit.rows  := vio_Height;
    Bit.width := vio_Width;
    Bit.flow  := TT_Flow_up;

    IF gray_level THEN
      Bit.cols := Bit.width
    ELSE
      Bit.cols := (Bit.width+7) DIV 8;

    Bit.size := Bit.rows * Bit.cols;
  END;

  {Autor: FREETYPE                                          }
  {Datum: 12.02.2000                                        }
  {Zweck: initialisiert die Raster Datenstruktur            }
  {IN   : nichts                                            }
  {OUT  : nichts                                            }
  PROCEDURE Clear_Data;
  BEGIN
    IF gray_level THEN
      fillchar( Bit.buffer^, Bit.size, gray_palette[0] )
    ELSE
      fillchar( Bit.buffer^, Bit.size, 0 );
  END;

BEGIN
  INIT_TTFAPI:=TT_INIT_FREETYPE;
  Set_Raster_Area;
  GetMem( Bit.buffer, Bit.size );
  Clear_Data;
  loaded:=FALSE; {noch keine Schrift geladen}
 {ein paar Anzeigeoptionen...}
  display_outline := true;
  hint_glyph      := true;
END;

{Autor: Markus Humm                                       }
{Datum: 12.02.2000                                        }
{Zweck: setzt die Parameter fr Font, Size etc.           }
{IN   : Fontdateiname, Size in pt. Antialias, Ausrichtung }
{OUT  : nichts                                            }
{Mu noch an den GFX-Treiber angepasst werden: wenn mono  }
{dann kann gray_level nie TRUE werden!!!                  }
PROCEDURE SET_TTFPARAMS(filename:STRING; pt_size:WORD; antialias:BOOLEAN);
BEGIN
  dateiname:=filename;
  res:=pt_size;
  gray_level:=antialias;
END;

{Autor: FREETYPE                                          }
{Datum: 12.02.2000                                        }
{Zweck: setzt die Parameter fr Schriftgre              }
{IN   : Schriftgre in Punkt                             }
{OUT  : TRUE, wenn erfolgreich                            }
FUNCTION Reset_Scale( res : Int ) : Boolean;
BEGIN
  error := TT_Set_Instance_CharSize( instance, res*64 );
  Reset_Scale := (error = TT_Err_Ok);
END;

{Autor: Markus Humm                                       }
{Datum: 12.02.2000                                        }
{Zweck: ldt eine Schriftart                              }
{IN   : Dateiname der Schrift, gesetzt durch SET_TTFPARAMS}
{OUT  : Fehlercode                                        }
FUNCTION OPEN_FONT(filename:STRING):WORD;
VAR w:WORD;
    cmcount,i:INTEGER; {Anzahl der Carmaps}
    plattform,encoding,cm:INTEGER; {fr Charmap}
BEGIN
 {zuvor geladene Schrift entladen!!!!}
  w:=TT_Open_Face( filename, face );
  IF w <> TT_ERR_OK THEN loaded:=FALSE
  ELSE
  BEGIN
    loaded:=TRUE;
    TT_Get_Face_Properties( face, props );
    w:=TT_New_Glyph( face, glyph );
    IF w = TT_ERR_OK THEN
    BEGIN
      w:=TT_New_Instance( face, instance );
     {VGA = 96x96 DPI (wird behauptet)}
      TT_Set_Instance_Resolutions( instance, 96, 96 );
      Reset_Scale( res );
     {Nach Unicode Charmap suchen und laden}
      cmcount:=TT_GET_CHARMAP_COUNT(face);
      cm:=-1;
      i:=1;
      WHILE (i < cmcount) AND NOT(plattform in [0,3]) DO
      BEGIN
        TT_GET_CHARMAP_ID(face,i,plattform,encoding);
        IF ((plattform = 3) AND (encoding = 1)) OR (plattform = 0) THEN
        BEGIN
          cm:=i;
        END;
        INC(i);
      END;
     {keine Unicode Tabelle gefunden}
      IF NOT(plattform in [0,3]) THEN
      BEGIN
        loaded:=FALSE;
        w:=TT_ERR_TABLE_MISSING;
      END
      ELSE
      BEGIN
        IF cm = -1 THEN cm:=cmcount;
        w:=TT_GET_CHARMAP(face,cm,charmap);
      END;
    END;
  END;
  OPEN_FONT:=w;
END;

{Autor: Markus Humm                                       }
{Datum: 12.02.2000                                        }
{Zweck: entldt eine geladene Schrift                     }
{IN   : nichts                                            }
{OUT  : nichts                                            }
PROCEDURE CLOSE_FONT;
BEGIN
  TT_Done_Glyph( glyph );
  TT_Close_Face( face );
END;

{Autor: Markus Humm                                       }
{Datum: 12.02.2000                                        }
{Zweck: deinitialisiert das TTF API                       }
{IN   : nichts                                            }
{OUT  : nichts                                            }
PROCEDURE DEINIT_TTFAPI;
BEGIN
  TT_Done_FreeType;
END;

{Autor: Markus Humm                                       }
{Datum: 12.02.2000                                        }
{Zweck: liefert den zu einem Zeichen gehrenden Glyph,    }
{       dazu wird von ASCII nach Unicode umgewandelt      }
{IN   : das Zeichen dessen Glyph gesucht wird             }
{OUT  : der Index des gesuchten Glyphen, oder 0 wenn kein }
{       entsprechender Glyph gefunden wurde               }
FUNCTION GET_GLYPHINDEX(c:CHAR):WORD;
var i:word;
BEGIN
  GET_GLYPHINDEX:=TT_CHAR_INDEX(charmap,ASCII2UNICODE(ORD(c)));
END;

{Autor: FREETYPE                                          }
{Datum: 12.02.2000                                        }
{Zweck: ldt ein Zeichen und scaliert dieses              }
{IN   : Index auf den Glyph, und ob hinting erfolgen soll }
{OUT  : Fehlercode                                        }
FUNCTION LoadTrueTypeChar( index : integer; hint  : boolean ) : TT_Error;
VAR j, load_flag : int;
    result       : TT_Error;

BEGIN
  IF hint THEN load_flag := TT_Load_Scale_Glyph or TT_Load_Hint_Glyph
          ELSE load_flag := TT_Load_Scale_Glyph;

  result := TT_Load_Glyph( instance,glyph,index,load_flag );

  LoadTrueTypeChar := result;
END;

{Autor: Markus Humm                                       }
{Datum: 12.02.2000                                        }
{Zweck: schreibt den String s von x,y ausgehend mit den   }
{       vorher eingestellten Parametern auf den Bildschirm}
{IN   : x,y Koordinate (linke obere Ecke des Textes) und  }
{       der auszugebende Text                             }
{OUT  : nichts                                            }
PROCEDURE OUTTEXTXY(x,y:INTEGER; s:STRING);
VAR i:BYTE;
    x1,y1:INTEGER;
BEGIN
  x1:=x; y1:=y;
  TT_Get_Instance_Metrics( instance, imetrics );

  FOR i:=1 TO LENGTH(s) DO
  BEGIN
    IF LOADTRUETYPECHAR(GET_GLYPHINDEX(s[i]),hint_glyph) = TT_ERR_OK THEN
    BEGIN
      TT_Get_Glyph_Metrics( glyph, metrics );

      IF gray_level THEN TT_Get_Glyph_Pixmap( glyph, Bit, x1*64, y1*64 )
      ELSE
        TT_Get_Glyph_Bitmap( glyph, Bit, x1*64, y1*64 );
    END
    ELSE
      WRITE(chr(7));

    INC(x1, (metrics.advance div 64) + 1);
  END;
END;

END.